@@ -5,7 +5,7 @@ import json
 import logging
 import time
 from asyncio.exceptions import TimeoutError
-from typing import Union, List
+from typing import Optional, Union, List
 
 import aiohttp
 from aiohttp.client_exceptions import (
@@ -22,6 +22,7 @@ from .exceptions import (
     MeshInvalidArguments,
     MeshInvalidCredentials,
     MeshInvalidInput,
+    MeshInvalidOutput,
     MeshNodeNotPrimary,
     MeshTimeoutError,
     MeshTooManyMatches,
@@ -35,7 +36,7 @@ _LOGGER_VERBOSE = logging.getLogger(f"{__name__}.verbose")
 def _get_action_index(action: str, payload: List[dict]) -> Union[int, None]:
     """Determine which index the supplied action is in the JNAP transaction results
 
-    The results are returned in a list in the order they were requested but we don't really
+    The results are returned in a list in the order they were requested, but we don't really
     now which order this will be because actions could be added to the payload dynamically.
 
     :param action: The JNAP action to look for
@@ -168,7 +169,7 @@ class Mesh:
         :param node: The node we should make a connection to
         :param password: password to use
         :param username: username to use; default admin
-        :param request_timeout: number of seconds to timeout the request; default 10s
+        :param request_timeout: number of seconds to time out the request; default 10s
         """
 
         if request_timeout is None:
@@ -187,8 +188,7 @@ class Mesh:
             const.ATTR_MESH_WAN_INFO: {},
         }
 
-        # noinspection HttpUrlsUsage
-        self.__api_url: str = f"http://{self.__mesh_attributes[const.ATTR_MESH_CONNECTED_NODE]}/JNAP/"
+        self.__api_url: str = self.__get_api_url(self.__mesh_attributes[const.ATTR_MESH_CONNECTED_NODE])
         self.__username: str = username
         self.__password: str = password
         self.__timeout: int = request_timeout
@@ -217,22 +217,39 @@ class Mesh:
 
         return ret
 
-    async def __async_make_request(self, action: str, payload=None) -> dict:
+    @staticmethod
+    def __get_api_url(host: str) -> str:
+        """Build the base URL for the API
+
+        :host: the host name of the node
+        """
+
+        # noinspection HttpUrlsUsage
+        return f"http://{host}/JNAP/"
+
+    async def __async_make_request(self, action: str, payload=None, node_address: Optional[str] = None) -> dict:
         """Execute the API request against the connected node.
 
         :param action: The JNAP action to execute
         :param payload: The relevant payload for the action
+        :param node_address: The node to send the request to (only valid for a subset of actions)
         :return: THe JSON response or raises an error if need be
         """
 
         _LOGGER_VERBOSE.debug(
             "URL: %s, Action: %s, Payload: %s, Timeout: %i",
-            self.__api_url,
+            node_address,
             action,
             json.dumps(payload),
             self.__timeout
         )
 
+        if node_address is not None and action != const.ACTION_JNAP_REBOOT:
+            raise MeshInvalidArguments
+
+        if node_address is None:
+            node_address = self.__api_url
+
         if payload is None:
             payload = []
 
@@ -242,7 +259,7 @@ class Mesh:
             if self._session.closed:  # session closed so recreate it
                 _LOGGER_VERBOSE.debug("Session was closed.")
                 self.__create_session()
-            resp = await self._session.post(url=self.__api_url, headers=headers, json=payload, timeout=self.__timeout)
+            resp = await self._session.post(url=node_address, headers=headers, json=payload, timeout=self.__timeout)
         except TimeoutError:
             raise MeshTimeoutError
         except (ClientConnectionError, ClientConnectorError,):
@@ -267,6 +284,8 @@ class Mesh:
                         err = None
                         if resp.get("result") == "_ErrorInvalidInput":
                             err = MeshInvalidInput(resp.get("error"))
+                        elif resp.get("result") == "_ErrorInvalidOutput":
+                            err = MeshInvalidOutput(resp.get("error"))
                         elif resp.get("result") == "_ErrorUnauthorized":
                             err = MeshInvalidCredentials
                         elif resp.get("result") == "_ErrorUnknownAction":
@@ -425,11 +444,6 @@ class Mesh:
                             node_firmware = node_firmware[0]
                         else:
                             node_firmware = {}
-                            _LOGGER.warning(
-                                "Node (%s) firmware status: %s",
-                                device.get("deviceID"),
-                                json.dumps(firmware_status)
-                            )
                     # endregion
                     n = Node(**device, **{"backhaul": device_backhaul, "updates": node_firmware})
                     devices.append(n)
@@ -647,7 +661,7 @@ class Mesh:
         """Delete a device from the device list on the mesh
 
         Supports deleting by device ID or device name.
-        Will error if neither the device ID or name are given.
+        Will error if neither the device ID nor name are given.
         Will error if multiple devices match the given name.
 
         :param kwargs: keyword arguments (device_id, device_name)
@@ -727,7 +741,7 @@ class Mesh:
     async def async_get_device_from_id(self, device_id: str, force_refresh: bool = False) -> Union[Device, Node]:
         """Get a Device or Node object based on the ID.
 
-        By default the stored information is used but you can refresh it from the API.
+        By default, the stored information is used, but you can refresh it from the API.
         Raises an error if the device is not found.
 
         :param device_id: The ID of the device to get details about
@@ -762,7 +776,7 @@ class Mesh:
         """To get a Device or Node object based on the MAC address.
 
         Searches through all known adapters on the device to find a match.
-        By default the stored information is used but you can refresh it from the API.
+        By default, the stored information is used, but you can refresh it from the API.
         Raises an error if the device is not found.
 
         :param mac_address: The MAC address to search for
@@ -884,11 +898,50 @@ class Mesh:
 
         return ret
 
+    async def async_reboot_node(self, node_name: str, force: bool = False) -> None:
+        """Reboot the given node
+
+        N.B. Rebooting the primary node will cause all nodes to reboot. If you're sure you want to
+        reboot the primary node, set the `force` parameter to `True`
+
+        :param node_name: the name of the node to restart
+        :param force: True to acknowledge the primary node, ignored for everything else
+        :return: None
+        """
+
+        _LOGGER.debug("Rebooting node: %s", node_name)
+
+        node_details: List[Node] = [
+            node
+            for node in self.nodes
+            if node.name.lower() == node_name.lower()
+        ]
+        if not node_details:
+            raise MeshDeviceNotFoundResponse
+
+        if node_details[0].type == const.NODE_TYPE_PRIMARY and not force:
+            # noinspection PyTypeChecker
+            raise MeshInvalidInput(f"{node_name} is a primary node. Use the force.")
+
+        node_ip = [
+            adapter.get("ip")
+            for adapter in node_details[0].network
+            if adapter.get("ip")
+        ]
+        if not node_ip:
+            # noinspection PyTypeChecker
+            raise MeshInvalidInput(f"{node_name}: no valid address found")
+
+        await self.__async_make_request(
+            action=const.ACTION_JNAP_REBOOT,
+            node_address=self.__get_api_url(host=node_ip[0])
+        )
+
     async def async_set_guest_wifi_state(self, state: bool) -> None:
         """Set the state of the guest Wi-Fi.
 
         The radios object is a required parameter for the API call but isn't handled in this method.
-        Instead a call is made to retrieve the existing settings and those are relayed back.  This assumes that
+        Instead, a call is made to retrieve the existing settings and those are relayed back.  This assumes that
         a guest network has been created in the official UI.
 
         :param state: True to enable, False to disable
@@ -905,7 +958,7 @@ class Mesh:
         """Set the state of the Parental Control feature.
 
         The rules are a required parameter for the API call but are not handled in this method.
-        Instead a call is made to retrieve the existing rules and those are relayed back.
+        Instead, a call is made to retrieve the existing rules and those are relayed back.
 
         :param state: True to enabled, False to disable
         :return: None
@@ -920,7 +973,7 @@ class Mesh:
     async def async_start_speedtest(self) -> None:
         """Instruct the mesh to carry out a Speedtest
 
-        A Speedtest is a long running task.  You should use the async_get_speedtest_state method to understand
+        A Speedtest is a long-running task.  You should use the async_get_speedtest_state method to understand
         the progress of the task.
 
         :return: None
